@@ -16,7 +16,8 @@
 #define _NVSHMEMX_H_
 
 /* NVRTC only compiles device code. Leave out host headers */
-#if !defined __CUDACC_RTC__ && !defined __clang_llvm_bitcode_lib__
+#if !defined __CUDACC_RTC__ && !defined __clang_llvm_bitcode_lib__ && \
+    !defined __NVSHMEM_NUMBA_SUPPORT__
 #include "host/nvshmemx_api.h"
 #include "device/tile/nvshmemx_tile_api.hpp"
 #include "device/nvshmemx_collective_launch_apis.h"
